@@ -91,8 +91,7 @@ class Defaulter(collections.defaultdict):
   https://docs.djangoproject.com/en/1.8/ref/templates/language/#variables
   """
   def __init__(self, init={}):
-    super(Defaulter, self).__init__(
-      Defaulter, {k: self.__defaulter(v) for k, v in init.items()})
+    super().__init__(Defaulter, {k: self.__defaulter(v) for k, v in init.items()})
 
   @classmethod
   def __defaulter(cls, obj):
@@ -104,13 +103,13 @@ class Defaulter(collections.defaultdict):
       return obj
 
   def __unicode__(self):
-    return super(Defaulter, self).__unicode__() if self else ''
+    return super().__unicode__() if self else ''
 
   __str__ = __unicode__
   __eq__ = collections.defaultdict.__eq__
 
   def __hash__(self):
-    return super(Defaulter, self).__hash__() if self else None.__hash__()
+    return super().__hash__() if self else None.__hash__()
 
 
 def activities_to_atom(activities, actor, title=None, request_url=None,
@@ -142,7 +141,7 @@ def activities_to_atom(activities, actor, title=None, request_url=None,
   for a in activities:
     _prepare_activity(a, reader=reader)
 
-  updated = (util.get_first(activities[0], 'object', default={}).get('published', '')
+  updated = (as1.get_object(activities[0]).get('published', '')
              if activities else '')
 
   if actor is None:
@@ -159,6 +158,7 @@ def activities_to_atom(activities, actor, title=None, request_url=None,
     updated=updated,
     VERBS_WITH_OBJECT=as1.VERBS_WITH_OBJECT,
     xml_base=xml_base,
+    as1=as1,
   )
 
 
@@ -181,6 +181,7 @@ def activity_to_atom(activity, xml_base=None, reader=True):
     mimetypes=mimetypes,
     VERBS_WITH_OBJECT=as1.VERBS_WITH_OBJECT,
     xml_base=xml_base,
+    as1=as1,
   )
 
 
@@ -349,7 +350,7 @@ def _prepare_activity(a, reader=True):
       Currently just includes location if True, not otherwise.
   """
   act_type = as1.object_type(a)
-  obj = util.get_first(a, 'object', default={})
+  obj = as1.get_object(a) or a
   primary = obj if (not act_type or act_type == 'post') else a
 
   # Render content as HTML; escape &s
@@ -377,7 +378,8 @@ def _prepare_activity(a, reader=True):
 
   # normalize actors
   for elem in a, obj:
-    _prepare_actor(elem.get('actor'))
+    elem['actor'] = as1.get_object(elem, 'actor')
+    _prepare_actor(elem['actor'])
 
   # normalize attachments, render attached notes/articles
   attachments = a.get('attachments') or obj.get('attachments') or []
@@ -387,7 +389,7 @@ def _prepare_activity(a, reader=True):
 
     if type == 'image':
       att['image'] = util.get_first(att, 'image')
-      image_atts.append(att['image'])
+      image_atts.append(att['image'] or att)
       continue
 
     image_urls_seen |= set(util.get_urls(att, 'image'))
