@@ -15,7 +15,7 @@ import urllib.parse
 from oauth_dropins.webutil import util
 from oauth_dropins.webutil.testutil import requests_response
 from oauth_dropins.webutil.util import json_dumps, json_loads
-from requests import HTTPError, RequestException
+from requests import HTTPError, JSONDecodeError, RequestException
 
 from . import source
 
@@ -146,13 +146,16 @@ class Mastodon(source.Source):
     if fn == util.requests_delete:
       return {}
 
-    content_type = resp.headers.get('Content-Type')
-    if not content_type or content_type.split(';')[0] != 'application/json':
-      logger.warning(f'Converting {resp.status_code} to 502 due to Content-Type {content_type} instead of application/json')
-      resp.status_code = 502
-      raise HTTPError(response=resp)
+    content_type = resp.headers.get('Content-Type', '')
+    if content_type.split(';')[0] != 'application/json':
+      # Truth Social returns text/plain;charset=UTF-8
+      logging.warning(f'Content-Type {content_type} is not application/json!')
 
-    return resp.json()
+    try:
+      return resp.json()
+    except JSONDecodeError as e:
+      resp.status_code = 502
+      raise HTTPError(e, response=resp)
 
   @classmethod
   def embed_post(cls, obj):
@@ -669,7 +672,7 @@ class Mastodon(source.Source):
       if num_media > MAX_MEDIA:
         videos = videos[:MAX_MEDIA]
         images = images[:max(MAX_MEDIA - len(videos), 0)]
-        logger.warning(f'Found %d media! Only using the first {MAX_MEDIA}: {videos + images!r}', num_media)
+        logger.warning(f'Found {num_media} media! Only using the first {MAX_MEDIA}: {videos + images!r}')
 
       if preview:
         media_previews = [
