@@ -63,6 +63,7 @@ HCARD = string.Template("""\
 LINK = string.Template('  <a class="u-$cls" href="$url"></a>')
 AS_TO_MF2_TYPE = {
   'event': ['h-event'],
+  'organization': ['h-card'],
   'person': ['h-card'],
   'place': ['h-card', 'p-location'],
 }
@@ -75,6 +76,7 @@ MF2_TO_AS_TYPE_VERB = {
   'like': ('activity', 'like'),
   'location': ('place', None),
   'note': ('note', None),
+  'org': ('organization', None),
   'person': ('person', None),
   'reply': ('comment', None),
   'repost': ('activity', 'share'),
@@ -187,7 +189,7 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   # for the object. for other verbs, the activity itself is the
   # interesting thing
   if obj_type == 'post':
-    primary = obj.get('object', {})
+    primary = as1.get_object(obj, 'object')
     obj_type = as1.object_type(primary) or default_object_type
   else:
     primary = obj
@@ -206,8 +208,7 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
 
   is_rsvp = obj_type in ('rsvp-yes', 'rsvp-no', 'rsvp-maybe')
   if (is_rsvp or obj_type == 'react') and obj.get('object'):
-    objs = obj['object']
-    in_reply_tos.extend(objs if isinstance(objs, list) else [objs])
+    in_reply_tos.extend(util.get_list(obj, 'object'))
 
   # maps objectType to list of objects
   attachments = defaultdict(list)
@@ -243,6 +244,7 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
       'uid': [obj.get('id') or ''],
       'numeric-id': [obj.get('numeric_id') or ''],
       'name': [name],
+      'org': [name] if obj_type == 'organization' else None,
       'nickname': [obj.get('username') or ''],
       ('note' if obj_type == 'person' else 'summary'): [summary],
       'url': (list(as1.object_urls(obj) or as1.object_urls(primary)) +
@@ -255,7 +257,8 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
       'size': sizes,
       'published': [obj.get('published', primary.get('published', ''))],
       'updated': [obj.get('updated', primary.get('updated', ''))],
-      'in-reply-to': util.trim_nulls([o.get('url') for o in in_reply_tos]),
+      'in-reply-to': util.trim_nulls([util.get_url(o) or o.get('id')
+                                      for o in in_reply_tos]),
       'author': [object_to_json(
         author, trim_nulls=False, default_object_type='person')],
       'location': [object_to_json(
@@ -304,7 +307,8 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   if obj_type == 'tag':
     ret['properties']['tag-of'] = util.get_urls(obj, 'target')
 
-  tags = obj.get('tags', []) or get_first(obj, 'object', {}).get('tags', [])
+  inner_obj = as1.get_object(obj)
+  tags = obj.get('tags', []) or inner_obj.get('tags', [])
   if not tags and obj_type == 'tag':
     tags = util.get_list(obj, 'object')
   ret['properties']['category'] = []
@@ -321,7 +325,7 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   if is_rsvp:
     ret['properties']['rsvp'] = [obj_type[len('rsvp-'):]]
   elif obj_type == 'invite':
-    invitee = object_to_json(obj.get('object'), trim_nulls=False,
+    invitee = object_to_json(inner_obj, trim_nulls=False,
                              default_object_type='person')
     ret['properties']['invitee'] = [invitee]
 
@@ -337,10 +341,10 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
       # single object, but it's useful to let it be a list, e.g. when a like has
       # multiple targets, e.g. a like of a post with original post URLs in it,
       # which brid.gy does.
-      objs = get_list(obj, 'object')
+      objs = as1.get_objects(obj)
       ret['properties'][prop + '-of'] = [
         # flatten contexts that are just a url
-        o['url'] if 'url' in o and set(o.keys()) <= set(['url', 'objectType'])
+        (o.get('url') or o.get('id')) if o.keys() <= set(['id', 'url', 'objectType'])
         else object_to_json(o, trim_nulls=False, entry_class='h-cite')
         for o in objs]
     else:
@@ -373,18 +377,23 @@ def object_to_json(obj, trim_nulls=True, entry_class='h-entry',
   return ret
 
 
-def json_to_object(mf2, actor=None, fetch_mf2=False):
+def json_to_object(mf2, actor=None, fetch_mf2=False, rel_urls=None):
   """Converts a single microformats2 JSON item to an ActivityStreams object.
 
   Supports h-entry, h-event, h-card, and other single item times. Does *not* yet
   support h-feed.
 
+  If `rel_urls` is provided, the returned `url` and `urls` fields will be
+  objects that may include `displayName` fields with the text or `title` from
+  the original HTML links.
+
   Args:
     mf2: dict, decoded JSON microformats2 object
     actor: optional author AS actor object. usually comes from a rel="author"
       link. if mf2 has its own author, that will override this.
     fetch_mf2: boolean, whether to fetch additional pages via HTTP if necessary,
       e.g. to determine authorship: https://indieweb.org/authorship
+    rel_urls: dict, optional `rel-urls` field from parsed mf2
 
   Returns:
     dict, ActivityStreams object
@@ -395,7 +404,6 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
   mf2 = copy.copy(mf2)
   props = mf2.setdefault('properties', {})
   prop = first_props(props)
-  rsvp = prop.get('rsvp')
 
   # convert author
   mf2_author = prop.get('author')
@@ -404,15 +412,8 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
   else:
     # the author h-card may be on another page. run full authorship algorithm:
     # https://indieweb.org/authorship
-    author = mf2util.find_author({'items': [mf2]}, hentry=mf2,
-                                 fetch_mf2_func=util.fetch_mf2 if fetch_mf2 else None)
-    if author:
-      author = {
-        'objectType': 'person',
-        'url': author.get('url'),
-        'displayName': author.get('name'),
-        'image': [{'url': author.get('photo')}],
-      }
+    author = find_author({'items': [mf2]}, hentry=mf2,
+                         fetch_mf2_func=util.fetch_mf2 if fetch_mf2 else None)
 
   if not author:
     author = actor
@@ -438,6 +439,7 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
     mf2_type = mf2util.post_type_discovery(without_photo)
 
   as_type, as_verb = MF2_TO_AS_TYPE_VERB.get(mf2_type, (None, None))
+  rsvp = get_text(prop.get('rsvp'))
   if rsvp:
     as_verb = f'rsvp-{rsvp}'
 
@@ -451,7 +453,14 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
     """Filter out relative and invalid URLs (mf2py gives absolute urls)."""
     return urllib.parse.urlparse(url).netloc
 
-  urls = props.get('url') and get_string_urls(props.get('url'))
+  # urls, with displayName if available in rel_urls
+  urls = []
+  for u in get_string_urls(props.get('url')):
+    rel = rel_urls.get(u, {}) if rel_urls else {}
+    urls.append({
+      'value': u,
+      'displayName': (rel.get('text') or rel.get('title') or '').strip(),
+    })
 
   # quotations: https://indieweb.org/quotation#How_to_markup
   attachments = [
@@ -506,8 +515,8 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
     'username': prop.get('nickname'),
     'summary': get_text(prop.get('summary') or prop.get('note')),
     'content': get_html(prop.get('content')),
-    'url': urls[0] if urls else None,
-    'urls': [{'value': u} for u in urls] if urls and len(urls) > 1 else None,
+    'url': urls[0]['value'] if urls else None,
+    'urls': urls if len(urls) > 1 or rel_urls else None,
     # image is special cased below, to handle alt
     'stream': [stream],
     'location': json_to_object(prop.get('location')),
@@ -559,9 +568,12 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
     objects = []
     for target in itertools.chain.from_iterable(
         props.get(field, []) for field in (
-          'follow-of', 'like', 'like-of', 'repost', 'repost-of', 'in-reply-to',
-          'invitee')):
-      t = json_to_object(target) if isinstance(target, dict) else {'url': target}
+          'follow-of', 'like-of', 'repost-of', 'in-reply-to', 'invitee')):
+      t = json_to_object(target) if isinstance(target, dict) else target
+      if rsvp:
+        if isinstance(t, str):
+          t = {'id': t}
+        t['objectType'] = 'event'
       # eliminate duplicates from redundant backcompat properties
       if t not in objects:
         objects.append(t)
@@ -582,7 +594,7 @@ def json_to_object(mf2, actor=None, fetch_mf2=False):
 
   else:
     obj.update({
-      'inReplyTo': [{'url': url} for url in in_reply_tos],
+      'inReplyTo': in_reply_tos,
       'author': author,
     })
 
@@ -603,7 +615,20 @@ def html_to_activities(html, url=None, actor=None, id=None):
   Returns:
     list of ActivityStreams activity dicts
   """
-  parsed = util.parse_mf2(html, url=url, id=id)
+  return json_to_activities(util.parse_mf2(html, url=url, id=id), actor=actor)
+
+
+def json_to_activities(parsed, actor=None):
+  """Converts parsed microformats2 JSON to ActivityStreams activities.
+
+  Args:
+    parsed: dict, parsed JSON microformats2 object
+    actor: optional author AS actor object for all activities. usually comes
+      from a rel="author" link.
+
+  Returns:
+    list of ActivityStreams activity dicts
+  """
   hfeed = mf2util.find_first_entry(parsed, ['h-feed'])
   items = hfeed.get('children', []) if hfeed else parsed.get('items', [])
 
@@ -613,7 +638,14 @@ def html_to_activities(html, url=None, actor=None, id=None):
     if 'h-entry' in types or 'h-event' in types or 'h-cite' in types:
       obj = json_to_object(item, actor=actor)
       obj['content_is_html'] = True
-      activities.append({'object': obj})
+      if obj.get('verb') or obj.get('objectType') == 'activity':
+        activities.append(obj)
+      else:
+        activities.append({
+          'objectType': 'activity',
+          'verb': 'post',
+          'object': obj,
+        })
 
   return activities
 
@@ -843,6 +875,9 @@ def hcard_to_html(hcard, parent_props=None):
 
   # extract first value from multiply valued properties
   props = hcard.get('properties', {})
+  if props.keys() == set(['uid']):
+    props['url'] = props.get('uid')
+
   prop = first_props(props)
   if not prop:
     return ''
@@ -967,26 +1002,24 @@ def render_content(obj, include_location=True, synthesize_content=True,
         'content' in obj):
       continue
 
-    targets = get_list(obj, 'object')
-    if not targets:
-      continue
+    for target in as1.get_objects(obj):
+      target_url = target.get('url') or target.get('id') or '#'
 
-    for target in targets:
       # sometimes likes don't have enough content to render anything
       # interesting
-      if 'url' in target and set(target) <= set(['url', 'objectType']):
-        content += f"<a href=\"{target.get('url')}\">{verb.lower()} this.</a>"
+      if target.keys() <= set(['id', 'url', 'objectType']):
+        content += f"<a href=\"{target_url}\">{verb.lower()} this.</a>"
 
       else:
-        author = target.get('author', target.get('actor', {}))
+        author = target.get('author') or target.get('actor') or {}
         # special case for twitter RT's
         if obj_type == 'share' and 'url' in obj and re.search(
             r'^https?://(?:www\.|mobile\.)?twitter\.com/', obj.get('url')):
-          content += f"RT <a href=\"{target.get('url', '#')}\">@{author.get('username')}</a> "
+          content += f"RT <a href=\"{target_url}\">@{author.get('username')}</a> "
         else:
           # image looks bad in the simplified rendering
           author = {k: v for k, v in author.items() if k != 'image'}
-          content += f"{verb} <a href=\"{target.get('url', '#')}\">{target.get('displayName', target.get('title', 'a post'))}</a> by {hcard_to_html(object_to_json(author, default_object_type='person'))}"
+          content += f"{verb} <a href=\"{target_url}\">{target.get('displayName', target.get('title', 'a post'))}</a> by {hcard_to_html(object_to_json(author, default_object_type='person'))}"
         content += render_content(target, include_location=include_location,
                                   synthesize_content=synthesize_content,
                                   white_space_pre=white_space_pre)
@@ -997,7 +1030,7 @@ def render_content(obj, include_location=True, synthesize_content=True,
 
   if render_attachments and obj.get('verb') == 'share':
     atts = [att for att in itertools.chain.from_iterable(
-              o.get('attachments', []) for o in util.get_list(obj, 'object'))
+              o.get('attachments', []) for o in as1.get_objects(obj))
             if att.get('objectType') not in ('note', 'article')]
     content += _render_attachments(atts, obj)
 
@@ -1076,16 +1109,19 @@ def find_author(parsed, **kwargs):
     parsed: dict, parsed mf2 object (ie return value from mf2py.parse())
     kwargs: passed through to mf2util.find_author()
   """
-  author = mf2util.find_author(parsed, 'http://123', **kwargs)
+  author = mf2util.find_author(parsed, **kwargs)
   if author:
     photo = author.get('photo')
-    if isinstance(photo, dict):
-      photo = photo.get('url') or photo.get('value')
-    return {
-      'displayName': author.get('name'),
+    alt = photo.get('alt') if isinstance(photo, dict) else None
+    return util.trim_nulls({
+      'objectType': 'person',
       'url': author.get('url'),
-      'image': {'url': photo},
-    }
+      'displayName': author.get('name'),
+      'image': [{
+        'url': get_text(photo),
+        'displayName': alt,
+      }],
+    })
 
 
 def get_title(mf2):
@@ -1164,8 +1200,10 @@ def maybe_linked_name(props):
   if name is not None:
     html = maybe_linked(name, url, linked_classname='p-name u-url',
                         unlinked_classname='p-name')
+  elif url:
+    html = util.pretty_link(url, attrs={'class': 'u-url'})
   else:
-    html = maybe_linked(url or '', url, linked_classname='u-url')
+    html = ''
 
   extra_urls = props.get('url', [])[1:]
   if extra_urls:
