@@ -1,6 +1,7 @@
 """Convert between ActivityStreams and RSS 2.0.
 
 RSS 2.0 spec: http://www.rssboard.org/rss-specification
+Feedgen docs: https://feedgen.kiesow.be/
 
 Apple iTunes Podcasts feed requirements:
 https://help.apple.com/itc/podcasts_connect/#/itc1723472cb
@@ -21,7 +22,7 @@ import feedparser
 import mf2util
 from oauth_dropins.webutil import util
 
-from . import microformats2
+from . import as1, microformats2
 
 logger = logging.getLogger(__name__)
 
@@ -81,11 +82,14 @@ def from_activities(activities, actor=None, title=None, feed_url=None,
   latest = None
   feed_has_enclosure = False
   for activity in activities:
-    obj = activity.get('object') or activity
-    if obj.get('objectType') == 'person':
+    obj = activity
+    if activity.get('verb') in ('create', 'post'):
+      obj = as1.get_object(activity)
+
+    if activity.get('objectType') == 'person':
       continue
 
-    item = fg.add_entry()
+    item = fg.add_entry(order='append')
     url = obj.get('url')
     id = obj.get('id') or url
     item.id(id)
