@@ -7,6 +7,25 @@ from oauth_dropins.webutil import testutil
 
 from .. import as1
 
+ACTOR = {
+  'id': 'tag:fake.com:444',
+  'displayName': 'Bob',
+  'url': 'https://plus.google.com/bob',
+  'image': {'url': 'https://bob/picture'},
+}
+NOTE = {
+  'objectType': 'note',
+  'id': 'tag:fake.com:my-note',
+  'url': 'http://fake.com/my-note',
+  'content': 'my note',
+  'published': '2012-02-22T20:26:41',
+}
+MENTION = copy.deepcopy(NOTE)
+MENTION['tags'] = [{
+  'objectType': 'mention',
+  'url': 'https://alice',
+  'displayName': 'Alice',
+}]
 LIKES = [{
   'verb': 'like',
   'author': {'id': 'tag:fake.com:person', 'numeric_id': '5'},
@@ -37,15 +56,33 @@ ACTIVITY = {
 }
 COMMENT = {
   'objectType': 'comment',
-  'content': 'foo bar',
-  'id': 'tag:fake.com:547822715231468_6796480',
+  'content': 'A ☕ reply',
+  'id': 'tag:fake.com:123456',
   'published': '2012-12-05T00:58:26+00:00',
-  'url': 'https://www.facebook.com/547822715231468?comment_id=6796480',
+  'url': 'https://fake.com/123456',
   'inReplyTo': [{
-    'id': 'tag:fake.com:547822715231468',
-    'url': 'https://www.facebook.com/547822715231468',
+    'id': 'tag:fake.com:123',
+    'url': 'https://fake.com/123',
   }],
 }
+POST = {
+  'objectType': 'activity',
+  'verb': 'post',
+  'id': 'tag:fake.com:123#post',
+  'object': COMMENT,
+}
+UPDATE = {
+  'objectType': 'activity',
+  'verb': 'update',
+  'id': 'tag:fake.com:123456#update',
+  'object': COMMENT,
+}
+DELETE_OF_ID = {
+  'objectType': 'activity',
+  'verb': 'delete',
+  'id': 'tag:fake.com:123456#delete',
+  'object': COMMENT['id'],
+}
 EVENT = {
   'id': 'tag:fake.com:246',
   'objectType': 'event',
@@ -103,13 +140,17 @@ LIKE = {
     'image': {'url': 'https://alice/picture'},
   },
 }
-RESHARER = {
-  'kind': 'plus#person',
-  'id': 'tag:fake.com:444',
-  'displayName': 'Bob',
-  'url': 'https://plus.google.com/bob',
-  'image': {'url': 'https://bob/picture'},
+FOLLOW = {
+  'id': 'tag:fake.com:333',
+  'objectType': 'activity',
+  'verb': 'follow',
+  'actor': ACTOR['id'],
+  'object': 'https://www.realize.be/',
 }
+FOLLOW_WITH_ACTOR = copy.deepcopy(FOLLOW)
+FOLLOW_WITH_ACTOR['actor'] = ACTOR
+FOLLOW_WITH_OBJECT = copy.deepcopy(FOLLOW)
+FOLLOW_WITH_OBJECT['object'] = ACTOR
 
 
 class As1Test(testutil.TestCase):
@@ -163,7 +204,7 @@ class As1Test(testutil.TestCase):
 
     gp_like = LIKE
     gp_like_edited = copy.deepcopy(gp_like)
-    gp_like_edited['author'] = RESHARER
+    gp_like_edited['author'] = ACTOR
 
     for before, after in (({}, {}),
                           ({'x': 1}, {'y': 2}),
@@ -193,6 +234,15 @@ class As1Test(testutil.TestCase):
       self.assertTrue(as1.activity_changed(before, after, log=True),
                       f'{before}\n{after}')
 
+  def test_activity_changed_in_reply_to_author_name(self):
+    first = copy.copy(COMMENT)
+    first['inReplyTo'][0]['author'] = copy.deepcopy(ACTOR)
+
+    second = copy.copy(COMMENT)
+    second['inReplyTo'][0]['author'] = copy.deepcopy(ACTOR)
+    second['inReplyTo'][0]['author']['displayName'] = 'other'
+    self.assertFalse(as1.activity_changed(first, second, log=True))
+
   def test_append_in_reply_to(self):
     fb_comment_before = copy.deepcopy(COMMENT)
     fb_comment_after_same = copy.deepcopy(fb_comment_before)
@@ -438,5 +488,65 @@ class As1Test(testutil.TestCase):
                    {'value': 'http://baz'},
           ],
         }),
+        (['https://www.jvt.me/img/profile.jpg'], {
+          'url': {
+            'value': 'https://www.jvt.me/img/profile.jpg',
+            'alt': "Jamie Tanna's profile image",
+          },
+        })
     ):
       self.assertEqual(expected, as1.object_urls(actor))
+
+  def test_get_ids(self):
+    for expected, obj in (
+        ([], {}),
+        ([], {'y': 'a'}),
+        ([], {'x': ''}),
+        ([], {'x': {}}),
+        ([], {'x': {'foo': 'bar'}}),
+        ([], {'x': [{}, '']}),
+        (['b'], {'x': 'b'}),
+        (['b'], {'y': 'a', 'x': 'b'}),
+        (['b'], {'x': ['', 'b']}),
+        (['b'], {'x': {'id': 'b'}}),
+        (['b'], {'x': {'url': 'b'}}),
+        (['b'], {'x': {'id': 'b', 'url': 'c'}}),
+        (['b'], {'x': [{}, 'b']}),
+        (['b'], {'x': [{'id': 'b'}, '']}),
+        (['b', 'd'], {'x': ['b', 'd']}),
+        (['b', 'd'], {'x': [{'id': 'b'}, {'url': 'd'}]}),
+    ):
+      with self.subTest(obj=obj):
+        self.assertEqual(expected, sorted(as1.get_ids(obj, 'x')), obj)
+
+  def test_get_object(self):
+    for expected, obj in (
+        ({}, None),
+        ({}, {}),
+        ({}, []),
+        ({}, {'f': None}),
+        ({}, {'f': {}}),
+        ({'id': 'x'}, {'f': 'x'}),
+        ({'y': 'z'}, {'f': {'y': 'z'}}),
+        ({'id': 'x'}, {'f': ['x']}),
+        ({'y': 'z'}, {'f': [{'y': 'z'}]}),
+    ):
+      with self.subTest(obj=obj):
+        self.assertEqual(expected, as1.get_object(obj, 'f'))
+
+  def test_get_objects(self):
+    for expected, obj in (
+        ([], None),
+        ([], {}),
+        ([], []),
+        ([], {'f': None}),
+        ([], {'f': {}}),
+        ([], {'f': []}),
+        ([{'id': 'x'}], {'f': 'x'}),
+        ([{'id': 'x'}], {'f': ['x']}),
+        ([{'y': 'z'}], {'f': {'y': 'z'}}),
+        ([{'y': 'z'}], {'f': [{'y': 'z'}]}),
+        ([{'id': 'x'}, {'y': 'z'}], {'f': ['x', {'y': 'z'}]}),
+    ):
+      with self.subTest(obj=obj):
+        self.assertEqual(expected, as1.get_objects(obj, 'f'))
