@@ -32,13 +32,25 @@ class ActivityStreams2Test(testutil.TestCase):
         with self.assertRaises(ValueError):
           fn(bad)
 
-    with self.assertRaises(ValueError):
-      as2.from_as1('z')
-
     with self.assertRaises(ValueError):
       # wrongly trying to parse mf2 JSON as AS2
       as2.to_as1({'type': ['h-card']})
 
+  def test_from_as1_misc(self):
+    as2.from_as1({
+      "objectType": "activity",
+      "verb": "follow",
+      "actor": {
+        "objectType": "person",
+        "image": [{
+          "url": {
+            "value": "https://www.jvt.me/img/profile.jpg",
+            "alt": "Jamie Tanna's profile image"
+          }
+        }],
+      },
+    })
+
   def test_to_as1_in_reply_to_string(self):
     self._test_to_as1_in_reply_to('http://x.y/z')
 
@@ -52,7 +64,7 @@ class ActivityStreams2Test(testutil.TestCase):
       'content': 'foo bar baz',
       'inReplyTo': in_reply_to,
     })
-    self.assertEqual([{'url': 'http://x.y/z'}], as1['inReplyTo'])
+    self.assertEqual(['http://x.y/z'], as1['inReplyTo'])
 
   def test_is_public(self):
     publics = list(PUBLICS)
