@@ -165,13 +165,39 @@ class AtomTest(testutil.TestCase):
       self.assert_multiline_in("""\
 <activity:verb>http://activitystrea.ms/schema/1.0/like</activity:verb>
 <activity:object>foo</activity:object>
+<published>2012-02-22</published>
+<updated>2013-10-25</updated>
 """, atom.activity_to_atom({
         'url': 'like-url',
         'objectType': 'activity',
         'verb': 'like',
         'object': obj,
+        'published': '2012-02-22',
+        'updated': '2013-10-25',
       }), ignore_blanks=True)
 
+  def test_activity_to_atom_bookmark(self):
+    got = atom.activity_to_atom({
+      'url': 'bookmark-url',
+      'objectType': 'activity',
+      'verb': 'post',
+      'object': {
+        'objectType': 'bookmark',
+        'targetUrl': 'http://orig',
+      },
+      'published': '2012-02-22',
+      'updated': '2013-10-25',
+    })
+    self.assert_multiline_in("""\
+<activity:object-type>http://activitystrea.ms/schema/1.0/bookmark</activity:object-type>
+""", got, ignore_blanks=True)
+    self.assert_multiline_in("""\
+<activity:verb>http://activitystrea.ms/schema/1.0/post</activity:verb>
+<published>2012-02-22</published>
+<updated>2013-10-25</updated>
+<link rel="self" type="application/atom+xml" href="bookmark-url" />
+""", got, ignore_blanks=True)
+
   def test_activity_to_atom_author_without_properties(self):
     """https://console.cloud.google.com/errors/CMPawqSghuDquwE"""
     self.assertIn('<title>foo</title>', atom.activity_to_atom({
@@ -423,6 +449,13 @@ quoted text
         ):
       self.assert_multiline_in(expected, out)
 
+  def test_empty_object_and_content(self):
+    out = atom.activities_to_atom([{
+      'object': None,
+      'content': 'fooey barry',
+    }], None)
+    self.assert_multiline_in('fooey barry', out)
+
   def test_attachments(self):
     got = atom.activities_to_atom([{'object': {'attachments': [
       {'objectType': 'note', 'url': 'http://p', 'content': 'note content'},
@@ -485,6 +518,11 @@ quoted tweet with photo
           'id': 'B',
           'url': 'https://twitter.com/B',
           'displayName': 'bee',
+        }, {
+          'id': 'https://www.w3.org/ns/activitystreams#Public'
+        }, {
+          'objectType': 'group',
+          'alias': '@public',
         }],
       },
     }], None)
@@ -793,6 +831,20 @@ going to Homebrew Website Club
     self.assertIn('<link rel="enclosure" href="http://pics/1.jpg?x&amp;y"', got)
     self.assertNotIn('<img class="u-photo" src="http://pics/2.jpg" alt="" />', got, got)
 
+  def test_image_attachment(self):
+    got = atom.activities_to_atom([{
+      'object': {
+        'content': 'foo bar',
+        'attachments': [{
+          'objectType': 'image',
+          'mediaType': 'image/jpeg',
+          'url': 'https://assets/img.jpeg',
+        }],
+      },
+    }], {})
+    self.assertIn('<img class="u-photo" src="https://assets/img.jpeg" alt="" />', got)
+    self.assertIn('<link rel="enclosure" href="https://assets/img.jpeg"', got)
+
   def test_context_in_reply_to(self):
     """context.inReplyTo should be translated to thr:in-reply-to."""
     activity = {
@@ -926,3 +978,18 @@ bar
 </div>
 </content>
 """, atom.activities_to_atom([activity], {}), ignore_blanks=True)
+
+  def test_bare_string_object(self):
+    activity = {
+      'objectType': 'activity',
+      'verb': 'rsvp-yes',
+      'object': 'https://2020.indieweb.org/east',
+    }
+    expected = """\
+<activity:verb>http://activitystrea.ms/schema/1.0/rsvp-yes</activity:verb>
+<activity:object>https://2020.indieweb.org/east</activity:object>
+"""
+    self.assert_multiline_in(expected, atom.activity_to_atom(activity, {}),
+                             ignore_blanks=True)
+    self.assert_multiline_in(expected, atom.activities_to_atom([activity], {}),
+                             ignore_blanks=True)
