@@ -89,6 +89,17 @@ class JsonFeedTest(testutil.TestCase):
       with self.assertRaises(TypeError):
         activities_to_jsonfeed(bad)
 
+  def test_activities_to_jsonfeed_string_object_actor(self):
+    self.assert_equals([{
+      'id': 'http://example.com/original/post',
+      'content_text': '',
+    }], activities_to_jsonfeed([{
+      "objectType": "activity",
+      "verb": "like",
+      "object": "http://example.com/original/post",
+      "actor": "http://example.com/author-456"
+    }])['items'])
+
   def test_jsonfeed_to_activities_attachment_extra_list(self):
     """Originally seen in:
     https://console.cloud.google.com/errors/CPyvpeH077rvIg
@@ -128,3 +139,14 @@ class JsonFeedTest(testutil.TestCase):
     """
     with self.assertRaises(ValueError):
       jsonfeed_to_activities({'items': [{'author': 'Ms. Foo'}]})
+
+  def test_convert_newlines_to_brs(self):
+    """https://github.com/snarfed/granary/issues/456"""
+    got = activities_to_jsonfeed([{
+      'content': 'foo\nbar\nbaz',
+    }])
+    self.assert_equals("""\
+foo<br />
+bar<br />
+baz
+""", got['items'][0]['content_html'])
