@@ -32,7 +32,7 @@ class Microformats2Test(testutil.TestCase):
 
   def test_ignore_h_as(self):
     """https://github.com/snarfed/bridgy/issues/635"""
-    obj = microformats2.json_to_object({'type': ['h-entry']})
+    obj = microformats2.json_to_object({'type': ['h-entry', 'h-as-article']})
     self.assertEqual('note', obj['objectType'])
 
   def test_html_content_and_summary(self):
@@ -779,7 +779,7 @@ Shared <a href="#">a post</a> by foo
     self.expect_requests_get('http://example.com', """
 <div class="h-card">
 <a class="p-name u-url" rel="me" href="/">Ms. ☕ Baz</a>
-<img class="u-photo" src="/my/pic" />
+<img class="u-photo" src="/my/pic" alt="my pic" />
 </div>
 """, response_headers={'content-type': 'text/html; charset=utf-8'})
     self.mox.ReplayAll()
@@ -791,7 +791,10 @@ Shared <a href="#">a post</a> by foo
         'url': 'http://example.com/',
         'objectType': 'person',
         'displayName': 'Ms. ☕ Baz',
-        'image': [{'url': 'http://example.com/my/pic'}],
+        'image': [{
+          'url': 'http://example.com/my/pic',
+          'displayName': 'my pic',
+        }],
       },
     }, microformats2.json_to_object({
       'type': ['h-entry'],
@@ -801,11 +804,125 @@ Shared <a href="#">a post</a> by foo
       },
     }, fetch_mf2=True))
 
+  def test_json_to_object_embedded_responses(self):
+    """Post with embedded responses as compound objects.
+
+    https://martymcgui.re/2020/07/15/what-we-talk-about-when-were-talking-about-webmentions/
+    """
+    self.assert_equals({
+      'objectType': 'activity',
+    }, microformats2.json_to_object({
+      'type': ['h-entry'],
+      'properties': {
+        'like': [{
+          'properties': {
+            'url': ['https://foo/bar'],
+          },
+        }, {
+          'properties': {
+            'url': ['https://baz/biff'],
+          },
+        }],
+        'rsvp': [{
+          'properties': {},
+        }],
+      },
+    }))
+
+  def test_json_to_object_rel_urls_actor_urls_text_title(self):
+    """https://github.com/snarfed/bridgy-fed/issues/331"""
+    self.assert_equals({
+      'objectType': 'person',
+      'url': 'http://one',
+      'urls': [{
+        'value': 'http://one',
+      }, {
+        'value': 'http://two',
+        'displayName': 'two title',
+      }, {
+        'value': 'http://three',
+      }, {
+        'value': 'http://four',
+        'displayName': 'four text',
+      }],
+    }, microformats2.json_to_object({
+      'type': ['h-card'],
+      'properties': {
+        'url': [
+          'http://one',
+          'http://two',
+          'http://three',
+          'http://four',
+        ],
+      },
+    }, rel_urls={
+      'http://one': {
+        'rels': ['me'],
+      },
+      'http://two': {
+        'title': 'two title ',
+      },
+      'http://four': {
+        'title': 'four title ',
+        'text': ' four text',
+      },
+    }))
+
+  def test_json_to_object_rel_urls_actor_urls_text_title_one_url(self):
+    """https://github.com/snarfed/bridgy-fed/issues/331"""
+    self.assert_equals({
+      'objectType': 'person',
+      'url': 'http://one',
+      'urls': [{
+        'value': 'http://one',
+        'displayName': 'one text',
+      }],
+    }, microformats2.json_to_object({
+      'type': ['h-card'],
+      'properties': {
+        'url': ['http://one'],
+      },
+    }, rel_urls={
+      'http://one': {
+        'rels': ['me'],
+        'text': ' one text\n ',
+      },
+    }))
+
+  def test_json_to_activities(self):
+    self.assert_equals([{
+      'objectType': 'activity',
+      'verb': 'post',
+      'object': {
+        'objectType': 'note',
+        'content': 'foo',
+        'content_is_html': True,
+      },
+    }, {
+      'objectType': 'activity',
+      'verb': 'share',
+      'object': 'http://orig/post',
+      'content_is_html': True,
+    }], microformats2.json_to_activities({
+      'items': [{
+        'type': ['h-entry'],
+        'properties': {
+          'content': ['foo'],
+        },
+      }, {
+        'type': ['h-entry'],
+        'properties': {
+          'repost-of': ['http://orig/post'],
+        },
+      }],
+    }))
+
   def test_find_author(self):
     self.assert_equals({
-    'displayName': 'my name',
-    'image': {'url': 'http://pic/ture'},
-  }, microformats2.find_author(mf2py.parse(doc="""\
+      'objectType': 'person',
+      'displayName': 'my name',
+      'image': [{'url': 'http://pic/ture'}],
+      }, microformats2.find_author(mf2py.parse(doc="""\
 <body class="h-entry">
 <div class="p-author h-card">
 <a class="p-name" href="http://li/nk">my name</a>
@@ -818,6 +935,18 @@ Shared <a href="#">a post</a> by foo
     self.assertEqual('', microformats2.hcard_to_html({}))
     self.assertEqual('', microformats2.hcard_to_html({'properties': {}}))
 
+  def test_hcard_to_html_only_id(self):
+    self.assert_multiline_equals("""\
+<span class="">
+  <data class="p-uid" value="https://foo/bar"></data>
+  <a class="u-url" href="https://foo/bar">foo/bar</a>
+</span>
+""", microformats2.hcard_to_html({
+  'properties': {
+    'uid': ['https://foo/bar'],
+  },
+}), ignore_blanks=True)
+
   def test_share_activity_to_json_html(self):
     """Should translate the full activity, not just the object."""
     share = {
@@ -863,7 +992,7 @@ Shared <a href="#">a post</a> by   <span class="h-card">
 <article class="h-entry">
   <span class="p-uid">http://localhost:3000/users/ryan#likes/7</span>
   <span class="p-author h-card">
-    <a class="u-url" href="http://localhost:3000/users/ryan">http://localhost:3000/users/ryan</a>
+    <a class="u-url" href="http://localhost:3000/users/ryan">localhost:3000/users/ryan</a>
   </span>
   <div class="e-content p-name">
   <a href="http://localhost/2017-10-01_mastodon-dev-6">likes this.</a>
@@ -904,12 +1033,16 @@ Shared <a href="#">a post</a> by   <span class="h-card">
 <div class="e-content p-name">foo bar<br />baz <br><br> baj</div>
 </article>"""
     activities = microformats2.html_to_activities(html)
-    self.assert_equals([{'object': {
-      'objectType': 'note',
-      'content': 'foo bar<br/>baz <br/><br/> baj',
-      'content_is_html': True,
-      'displayName': 'foo bar\nbaz \n\n baj',
-    }}], activities)
+    self.assert_equals([{
+      'objectType': 'activity',
+      'verb': 'post',
+      'object': {
+        'objectType': 'note',
+        'content': 'foo bar<br/>baz <br/><br/> baj',
+        'content_is_html': True,
+        'displayName': 'foo bar\nbaz \n\n baj',
+      },
+    }], activities)
 
   def test_html_to_activities_filters_items(self):
     """Check that we omit h-cards inside h-feeds."""
