@@ -4,14 +4,16 @@
 import copy
 
 from mox3 import mox
+from oauth_dropins import reddit as oauth_reddit
 from oauth_dropins.webutil import testutil
 from oauth_dropins.webutil import util
 
 from granary import reddit
 
 import praw
-import praw.models
+from praw.models import Subreddit, User
 from praw.models.comment_forest import CommentForest
+from praw.models.listing.mixins.redditor import SubListing
 from prawcore.exceptions import NotFound
 
 
@@ -165,13 +167,15 @@ class RedditTest(testutil.TestCase):
 
   def setUp(self):
     super(RedditTest, self).setUp()
+    oauth_reddit.REDDIT_APP_KEY = oauth_reddit.REDDIT_APP_LOCAL = 'foo'
     self.reddit = reddit.Reddit('token-here')
-    self.api = self.reddit.reddit_api = self.mox.CreateMockAnything(praw.Reddit)
+    self.api = self.reddit.api = self.mox.CreateMockAnything(praw.Reddit)
 
-    self.submission_selftext = FakeSubmission(FakeRedditor())
-    self.comment = FakeComment(self.submission_selftext, FakeRedditor())
+    self.redditor = FakeRedditor()
+    self.submission_selftext = FakeSubmission(self.redditor)
+    self.comment = FakeComment(self.submission_selftext, self.redditor)
 
-    self.submission_link = FakeSubmission(FakeRedditor())
+    self.submission_link = FakeSubmission(self.redditor)
     self.submission_link.selftext = ''
     self.submission_link.url = 'https://reddit.com/ezv3f2'
 
@@ -189,19 +193,18 @@ class RedditTest(testutil.TestCase):
     self.assert_equals(MISSING_OBJECT, self.reddit.praw_to_actor(suspended))
 
   def test_praw_to_actor(self):
-    self.assert_equals(ACTOR, self.reddit.praw_to_actor(FakeRedditor()))
+    self.assert_equals(ACTOR, self.reddit.praw_to_actor(self.redditor))
 
   def test_broken_praw_to_actor(self):
     self.assert_equals(MISSING_OBJECT, self.reddit.praw_to_actor(util.Struct()))
 
   def test_praw_to_actor_no_subreddit(self):
-    redditor = FakeRedditor()
-    redditor.subreddit = None
+    self.redditor.subreddit = None
     expected = copy.deepcopy(ACTOR)
     del expected['description']
     del expected['urls']
     expected['url'] = 'https://reddit.com/user/bonkerfield/'
-    self.assert_equals(expected, self.reddit.praw_to_actor(redditor))
+    self.assert_equals(expected, self.reddit.praw_to_actor(self.redditor))
 
   def test_praw_to_comment(self):
     self.assert_equals(COMMENT_OBJECT,
@@ -230,14 +233,26 @@ class RedditTest(testutil.TestCase):
       'lhzukq',
       self.reddit.post_id('https://www.reddit.com/r/xyz/comments/lhzukq/abc/'))
 
-  # TODO
-  # def test_get_activities_default_user(self):
-  #   self.api.user.me().submissions.new(limit=None).AndReturn(
-  #     [self.submission_selftext, self.submission_link])
-  #   self.mox.ReplayAll()
+  def test_get_activities_user_id(self):
+    self.api.redditor('plfff').AndReturn(self.redditor)
+    self.redditor.submissions = self.mox.CreateMock(SubListing)
+    self.redditor.submissions.new(limit=None).AndReturn(
+      [self.submission_selftext, self.submission_link])
+    self.mox.ReplayAll()
 
-  #   self.assert_equals([ACTIVITY_WITH_SELFTEXT, ACTIVITY_WITH_LINK],
-  #                      self.reddit.get_activities())
+    self.assert_equals([ACTIVITY_WITH_SELFTEXT, ACTIVITY_WITH_LINK],
+                       self.reddit.get_activities(user_id='plfff'))
+
+  def test_get_activities_default_user(self):
+    self.api.user = self.mox.CreateMock(User)
+    self.api.user.me().AndReturn(self.redditor)
+    self.redditor.submissions = self.mox.CreateMock(SubListing)
+    self.redditor.submissions.new(limit=None).AndReturn(
+      [self.submission_selftext, self.submission_link])
+    self.mox.ReplayAll()
+
+    self.assert_equals([ACTIVITY_WITH_SELFTEXT, ACTIVITY_WITH_LINK],
+                       self.reddit.get_activities())
 
   def test_get_activities_activity_id(self):
     self.api.submission(id='abc').AndReturn(self.submission_selftext)
@@ -247,7 +262,7 @@ class RedditTest(testutil.TestCase):
                        self.reddit.get_activities(activity_id='abc'))
 
   def test_get_activities_search_query(self):
-    subreddit = self.mox.CreateMock(praw.models.Subreddit)
+    subreddit = self.mox.CreateMock(Subreddit)
     self.mox.StubOutWithMock(self.api, 'subreddit')
     self.api.subreddit('all').AndReturn(subreddit)
     subreddit.search('foo bar', sort='new', limit=None).AndReturn(
@@ -269,8 +284,62 @@ class RedditTest(testutil.TestCase):
       [ACTIVITY_WITH_COMMENT],
       self.reddit.get_activities(activity_id='ezv3f2', fetch_replies=True))
 
+  def test_get_activities_cache_comments(self):
+    # get first_activities() call, fetches comments
+    self.submission_selftext.num_comments = 1
+    for _ in range(2):
+      self.api.submission(id='ezv3f2').AndReturn(self.submission_selftext)
+
+    comments = CommentForest(self.submission_selftext, comments=[self.comment])
+    self.submission_selftext.comments = comments
+    self.mox.StubOutWithMock(comments, 'replace_more')
+    comments.replace_more()
+
+    # second call, comment count is unchanged, skips comment fetch
+    for _ in range(2):
+      self.api.submission(id='ezv3f2').AndReturn(self.submission_selftext)
+
+    # third call, comment count is different, skips comment fetch
+    other_sub = FakeSubmission(self.redditor)
+    other_sub.num_comments = 2
+    other_sub.comments = comments
+    for _ in range(2):
+      self.api.submission(id='ezv3f2').AndReturn(other_sub)
+    comments.replace_more()
+
+    # fourth call, comment count is unchanged, skips comment fetch
+    for _ in range(2):
+      self.api.submission(id='ezv3f2').AndReturn(other_sub)
+
+    self.mox.ReplayAll()
+    cache = {}
+    for num_comments in (1, 2):
+      # not cached
+      self.assert_equals(
+        [ACTIVITY_WITH_COMMENT],
+        self.reddit.get_activities(activity_id='ezv3f2', fetch_replies=True, cache=cache))
+      self.assert_equals(num_comments, cache['ARR ezv3f2'])
+
+      # not cached
+      self.assert_equals(
+        [ACTIVITY_WITH_SELFTEXT],
+        self.reddit.get_activities(activity_id='ezv3f2', fetch_replies=True, cache=cache))
+      self.assert_equals(num_comments, cache['ARR ezv3f2'])
+
   def test_get_comment(self):
     self.api.comment(id='xyz').AndReturn(self.comment)
     self.mox.ReplayAll()
 
     self.assert_equals(COMMENT_OBJECT, self.reddit.get_comment('xyz'))
+
+  def test_get_actor(self):
+    self.api.redditor('schnarfed').AndReturn(self.redditor)
+    self.mox.ReplayAll()
+
+    self.assert_equals(ACTOR, self.reddit.get_actor('schnarfed'))
+
+  def test_get_actor_missing(self):
+    self.api.redditor('schnarfed').AndReturn(self.comment)
+    self.mox.ReplayAll()
+
+    self.assert_equals(MISSING_OBJECT, self.reddit.get_actor('schnarfed'))
