@@ -23,6 +23,7 @@ from oauth_dropins.webutil import util
 from oauth_dropins.webutil.util import json_dumps, json_loads
 from requests import RequestException
 
+from . import as1
 from . import source
 
 logger = logging.getLogger(__name__)
@@ -198,7 +199,7 @@ class Twitter(source.Source):
                               activity_id=None, start_index=0, count=0,
                               etag=None, min_id=None, cache=None,
                               fetch_replies=False, fetch_likes=False,
-                              fetch_shares=False, include_shares=False,
+                              fetch_shares=False, include_shares=True,
                               fetch_events=False, fetch_mentions=False,
                               search_query=None, scrape=False, **kwargs):
     """Fetches posts and converts them to ActivityStreams activities.
@@ -338,13 +339,9 @@ class Twitter(source.Source):
         else:
           raise
 
-    # batch get memcached counts of favorites and retweets for all tweets
-    cached = {}
-    if cache is not None:
-      keys = itertools.product(('ATR', 'ATF'), [t['id_str'] for t in tweets])
-      cached = cache.get_multi(f'{prefix} {id}' for prefix, id in keys)
-    # only update the cache at the end, in case we hit an error before then
-    cache_updates = {}
+    if cache is None:
+      # for convenience, throwaway object just for this method
+      cache = {}
 
     if fetch_shares:
       retweet_calls = 0
@@ -372,7 +369,7 @@ class Twitter(source.Source):
         # returns the original tweets, not the retweets or their authors.
         id = tweet['id_str']
         count = tweet.get('retweet_count')
-        if count and count != cached.get('ATR ' + id):
+        if count and count != cache.get('ATR ' + id):
           url = API_RETWEETS % id
           if min_id is not None:
             url = util.add_query_params(url, {'since_id': min_id})
@@ -391,7 +388,7 @@ class Twitter(source.Source):
               raise
 
           retweet_calls += 1
-          cache_updates['ATR ' + id] = count
+          cache['ATR ' + id] = count
 
     if not include_shares:
       tweets = [t for t in tweets if not t.get('retweeted_status')]
@@ -412,7 +409,7 @@ class Twitter(source.Source):
       for tweet, activity in zip(tweets, tweet_activities):
         id = tweet['id_str']
         count = tweet.get('favorite_count')
-        if self.is_public(activity) and count and count != cached.get('ATF ' + id):
+        if as1.is_public(activity) and count and count != cache.get('ATF ' + id):
           try:
             resp = util.requests_get(SCRAPE_LIKES_URL % id,
                                      headers=self.scrape_headers)
@@ -424,13 +421,11 @@ class Twitter(source.Source):
           likes = [self._make_like(tweet, author) for author in
                    resp.json().get('globalObjects', {}).get('users', {}).values()]
           activity['object'].setdefault('tags', []).extend(likes)
-          cache_updates['ATF ' + id] = count
+          cache['ATF ' + id] = count
 
     activities += tweet_activities
     response = self.make_activities_base_response(activities)
     response.update({'total_count': total_count, 'etag': etag})
-    if cache_updates and cache is not None:
-      cache.set_multi(cache_updates)
     return response
 
   def fetch_replies(self, activities, min_id=None):
@@ -789,17 +784,17 @@ class Twitter(source.Source):
     preview_content = HASHTAG_RE.sub(
       r'\1<a href="https://twitter.com/hashtag/\2">#\2</a>', preview_content)
 
-    if type == 'activity' and verb == 'like':
+    if type == 'activity' and verb in ('like', 'favorite'):
       if not base_url:
         return source.creation_result(
           abort=True,
           error_plain='Could not find a tweet to like.',
-          error_html='Could not find a tweet to <a href="http://indiewebcamp.com/favorite">favorite</a>. '
+          error_html='Could not find a tweet to <a href="http://indiewebcamp.com/like">like</a>. '
           'Check that your post has a like-of link to a Twitter URL or to an original post that publishes a '
           '<a href="http://indiewebcamp.com/rel-syndication">rel-syndication</a> link to Twitter.')
 
       if preview:
-        preview_description += f"""<span class="verb">favorite</span>
+        preview_description += f"""<span class="verb">like</span>
 <a href="{base_url}">this tweet</a>:
 {self.embed_post(base_obj)}"""
         return source.creation_result(description=preview_description)
@@ -856,7 +851,7 @@ class Twitter(source.Source):
           images = images[:MAX_MEDIA]
           logger.warning(f'Found {num} photos! Only using the first {MAX_MEDIA}: {images}')
         preview_content += '<br /><br />' + ' &nbsp; '.join(
-          f"<img src=\"{img.get('url')}\" alt=\"{img.get('displayName', '')}\" />"
+          f"<img src=\"{img.get('url')}\" alt=\"{util.ellipsize(img.get('displayName', ''), words=1000, chars=MAX_ALT_LENGTH)}\" />"
                                          for img in images)
         if not preview:
           ret = self.upload_images(images)
@@ -932,7 +927,7 @@ class Twitter(source.Source):
 
       alt = image.get('displayName')
       if alt:
-        alt = util.ellipsize(alt, chars=MAX_ALT_LENGTH)
+        alt = util.ellipsize(alt, words=1000, chars=MAX_ALT_LENGTH)
         headers = twitter_auth.auth_header(
           API_MEDIA_METADATA, self.access_token_key, self.access_token_secret, 'POST')
         resp = util.requests_post(
